package cn.aibreeze.exam.api;

import cn.aibreeze.exam.dto.*;
import cn.aibreeze.exam.entity.ElehKnowledgeDo;
import cn.aibreeze.exam.entity.ElehKnowledgeTagDo;
import cn.aibreeze.exam.entity.ElehSessionQaRecordDo;
import cn.aibreeze.exam.facade.ExamAppFacade;
import cn.aibreeze.exam.mapper.ElehKnowledgeMapper;
import cn.aibreeze.exam.mapper.ElehKnowledgeTagMapper;
import cn.aibreeze.exam.mapper.ElehSessionQaRecordMapper;
import cn.aibreeze.exam.util.AliyunNLSFacade;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import io.micrometer.common.util.StringUtils;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.compress.utils.Lists;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.*;

import java.text.DecimalFormat;
import java.util.Date;
import java.util.List;
import java.util.Objects;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;


@RestController
@Slf4j
@RequestMapping(value = "/exam")
@RequiredArgsConstructor
public class ExamController {


    private final ElehKnowledgeMapper knowledgeMapper;

    private final ElehKnowledgeTagMapper tagMapper;

    private final ElehSessionQaRecordMapper qaRecordMapper;

    private final StringRedisTemplate redisTemplate;

    private final AliyunNLSFacade aliyunNLSFacade;

    private final ExamAppFacade examAppFacade;

    private static String EXAM_REDIS_QUESTION_KEY = "EXAM_MASTER_QUESTION:%d:%s:%s";

    private static String EXAM_REDIS_SESSION_KEY = "EXAM_MASTER_SESSION:%d:%s:%s";

    @GetMapping(value = "/list")
    public String list() {
        LambdaQueryWrapper<ElehKnowledgeDo> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(ElehKnowledgeDo::getTenant, 16);
        List<ElehKnowledgeDo> elehKnowledgeDos = knowledgeMapper.selectList(queryWrapper);
        return JSONObject.toJSONString(elehKnowledgeDos);
    }

    @GetMapping(value = "/get_current_question")
    public ApiResponse<GetQuestionResponseResult> getCurrentQuestion(@RequestParam("tenant") Integer tenant,
                                                                    @RequestParam("session_id") String sessionId,
                                                                    @RequestParam("user_id") String userId) {

        GetQuestionResponseResult result = new GetQuestionResponseResult();

        String redisKey = String.format(EXAM_REDIS_QUESTION_KEY, tenant, sessionId, userId);

        String currentQuestion = redisTemplate.opsForValue().get(redisKey);

        log.info("获取当前出题，rediskey = {}, value = {}", redisKey, currentQuestion);
        if(StringUtils.isNotEmpty(currentQuestion)) {
            ElehKnowledgeDo knowledgeDo = knowledgeMapper.selectById(tenant, Long.valueOf(currentQuestion));
            if(Objects.nonNull(knowledgeDo)) {
                ElehKnowledgeTagDo tagDo = tagMapper.selectById(knowledgeDo.getTagId());

                result.setQuestion(knowledgeDo.getQuestion());
                result.setAnswer(knowledgeDo.getAnswer());
                result.setProject(tagDo.getName());
                result.setCode(tagDo.getCode());
                result.setQuestionId(knowledgeDo.getId());
                result.setNext(true);
            }
        }

        return ApiResponse.ok(result);
    }

    @GetMapping(value = "/get_question")
    public ApiResponse<GetQuestionResponseResult> getQuestion(@RequestParam("tenant") Integer tenant,
                                                              @RequestParam("code") String code,
                                                              @RequestParam("session_id") String sessionId,
                                                              @RequestParam("user_id") String userId) {

        GetQuestionResponseResult result = new GetQuestionResponseResult();

        // 根据分类获取考题
        ElehKnowledgeTagDo tagDo = tagMapper.selectByCode(tenant, code);
        if(Objects.nonNull(tagDo)) {
            Long maxQuestionId = qaRecordMapper.selectSessionMaxQuestionId(tenant, sessionId, userId);
            maxQuestionId = Objects.nonNull(maxQuestionId) ? maxQuestionId : 0;
            ElehKnowledgeDo knowledgeDo = knowledgeMapper.selectQuestion(tenant, tagDo.getId(), maxQuestionId);
            if(Objects.nonNull(knowledgeDo)) {
                result.setQuestion(knowledgeDo.getQuestion());
                result.setAnswer(knowledgeDo.getAnswer());
                result.setProject(tagDo.getName());
                result.setCode(tagDo.getCode());
                result.setQuestionId(knowledgeDo.getId());
                result.setNext(true);

                // 存储当前会话考试信息
                String redisKey = String.format(EXAM_REDIS_QUESTION_KEY, tenant, sessionId, userId);
                redisTemplate.opsForValue().set(redisKey, result.getQuestionId()+"", 10, TimeUnit.MINUTES);

                log.info("出题结果：result = {}, rediskey = {}", JSONObject.toJSONString(result), redisKey);
            }
        }

        return ApiResponse.ok(result);
    }

    @PostMapping(value = "update_question")
    public ApiResponse<String> updateQuestion(@RequestParam("tenant") Integer tenant,
                                              @RequestParam("session_id") String sessionId,
                                              @RequestParam("user_id") String userId,
                                              @RequestParam("question_id") Long questionId,
                                              @RequestParam(value = "answer", required = false) String answer,
                                              @RequestParam(value = "score", required = false) Double score,
                                              @RequestParam(value = "score_criteria", required = false) String scoreCriteria) {


        ElehSessionQaRecordDo recordDo = new ElehSessionQaRecordDo();
        recordDo.setQuestionId(questionId);
        recordDo.setAnswer(answer);
        recordDo.setTenant(tenant);
        recordDo.setSessionId(sessionId);
        recordDo.setUserId(userId);
        recordDo.setScoreCriteria(scoreCriteria);
        recordDo.setScore(score);
        recordDo.setCreatedAt(new Date());
        qaRecordMapper.insert(recordDo);

        return ApiResponse.ok("SUCCESS");
    }

    @GetMapping(value = "/get_final_result")
    public ApiResponse<SessionExamResult> getFinalResult(@RequestParam("tenant") Integer tenant,
                                                         @RequestParam("session_id") String sessionId,
                                                         @RequestParam("user_id") String userId) {

        String avgScore = "";
        Double totalScore = 0D;
        int examNum = 0;
        List<SingleQaResult> qaResultList = Lists.newArrayList();

        LambdaQueryWrapper<ElehSessionQaRecordDo> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(ElehSessionQaRecordDo::getTenant, tenant);
        queryWrapper.eq(ElehSessionQaRecordDo::getSessionId, sessionId);
        queryWrapper.eq(ElehSessionQaRecordDo::getUserId, userId);
        List<ElehSessionQaRecordDo> qaRecordDos = qaRecordMapper.selectList(queryWrapper);
        if(!CollectionUtils.isEmpty(qaRecordDos)) {
            totalScore = qaRecordDos.stream().mapToDouble(ElehSessionQaRecordDo::getScore).sum();
            examNum = qaRecordDos.size();

            DecimalFormat df = new DecimalFormat("#.##");
            avgScore = df.format(totalScore / examNum);

            // 数据转换
            qaResultList = qaRecordDos.stream().map(v -> {

                ElehKnowledgeDo knowledgeDo = knowledgeMapper.selectById(v.getTenant(), v.getQuestionId());

                SingleQaResult qa = new SingleQaResult();
                qa.setQuestion(knowledgeDo.getQuestion());
                qa.setAnswer(knowledgeDo.getAnswer());
                qa.setUserAnswer(v.getAnswer());
                qa.setScore(v.getScore());
                qa.setScoreCriteria(v.getScoreCriteria());
                return qa;
            }).collect(Collectors.toList());
        }
        SessionExamResult result = new SessionExamResult();
        result.setAvgScore(avgScore);
        result.setTotalScore(totalScore);
        result.setExamNum(examNum);
        result.setQaResultList(qaResultList);
        return ApiResponse.ok(result);
    }

    @GetMapping(value = "/voice_to_content")
    public ApiResponse<VoiceToContentResult> voiceToContent(@RequestParam(value = "tenant", required = false) Integer tenant,
                                                            @RequestParam(value = "session_id", required = false) String sessionId,
                                                            @RequestParam(value = "user_id",required = false) String userId,
                                                            @RequestParam("file_url") String fileUrl) {

        VoiceToContentResult resultContent = new VoiceToContentResult();

        P2PResponse<String> textFromMedia = aliyunNLSFacade.getTextFromMedia(fileUrl);
        if(textFromMedia.isSuccess()) {
            boolean asrDone = false;
            do {
                P2PResponse<String> asrResult = aliyunNLSFacade.getAsrResult(textFromMedia.getBody());
                asrDone = asrResult.isSuccess();
                resultContent.setContent(asrResult.getBody());
                if(!asrDone) {
                    try {
                        Thread.sleep(3000L);
                    } catch (InterruptedException e) {
                        throw new RuntimeException(e);
                    }
                }
            } while (!asrDone);
        }

        return ApiResponse.ok(resultContent);
    }


    @PostMapping(value = "/run4score")
    public ApiResponse<String> run4Score(@RequestParam(value = "tenant") Integer tenant,
                                         @RequestParam(value = "scene") String scene,
                                         @RequestParam(value = "session_id") String sessionId,
                                         @RequestParam(value = "user_id") String userId,
                                         @RequestParam(value = "question") String question,
                                         @RequestParam(value = "answer") String answer,
                                         @RequestParam(value = "user_answer") String userAnswer) {


        String redisKey = String.format(EXAM_REDIS_SESSION_KEY, tenant, sessionId, userId);

        SingleQaResult qa = new SingleQaResult();
        qa.setUserId(userId);
        qa.setSessionId(sessionId);
        qa.setUserAnswer(userAnswer);
        qa.setQuestion(question);
        qa.setAnswer(answer);
        qa.setScene(scene);

        log.info("PUT REDIS KEY = {}, REDIS VALUE = {}", redisKey, JSONObject.toJSONString(qa));

        redisTemplate.opsForValue().set(redisKey, JSONObject.toJSONString(qa), 30, TimeUnit.MINUTES);

        examAppFacade.run4Score(scene, sessionId, userId, question, answer, userAnswer);

        return ApiResponse.ok("ok");
    }

    @GetMapping(value = "/get_run4score_info")
    public ApiResponse<SingleQaResult> getRun4ScoreInfo(@RequestParam(value = "tenant") Integer tenant,
                                                        @RequestParam(value = "session_id") String sessionId,
                                                        @RequestParam(value = "user_id") String userId) {


        String redisKey = String.format(EXAM_REDIS_SESSION_KEY, tenant, sessionId, userId);

        String redisValue = redisTemplate.opsForValue().get(redisKey);


        log.info("GET REDIS KEY = {}, REDIS VALUE = {}", redisKey, redisValue);

        if(StringUtils.isNotEmpty(redisValue)) {
            SingleQaResult singleQaResult = JSONObject.parseObject(redisValue, SingleQaResult.class);
            // 获取当天考题
            String questionRedisKey = String.format(EXAM_REDIS_QUESTION_KEY, tenant, sessionId, userId);
            singleQaResult.setQuestionId(Long.valueOf(redisTemplate.opsForValue().get(questionRedisKey)));

            log.info("get_run4score_info, response = {}", JSONObject.toJSONString(singleQaResult));

            return ApiResponse.ok(singleQaResult);
        }
        return ApiResponse.fail("no cache data.");
    }
}
