package cn.aibreeze.exam.api;

import cn.aibreeze.exam.config.FileUploadResult;
import cn.aibreeze.exam.dto.ApiResponse;
import cn.aibreeze.exam.service.PrivateFileService;
import cn.aibreeze.exam.util.PeerException;
import cn.hutool.core.io.FileUtil;
import com.google.common.collect.Lists;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

@Slf4j
@RestController
@RequestMapping(value = "/common")
@RequiredArgsConstructor
public class UploadController {

    private final PrivateFileService fileService;


    @PostMapping(value = "/upload")
    public ApiResponse<List<String>> upload(@RequestParam("files") MultipartFile[] files) {
        List<String> urls = Lists.newArrayList();
        // 判断文件数组长度
        if(files.length <= 0){
            return ApiResponse.fail("请选择文件");
        }
        for(MultipartFile file : files){
            File tmp = null;
            try {
                tmp = File.createTempFile("upload", "");
                file.transferTo(tmp);
                String key = UUID.randomUUID().toString() + "." + FileUtil.extName(file.getOriginalFilename());
                urls.add(fileService.upload(key, tmp));
            } catch (Exception e) {
                log.error("upload error,", e);
                throw new PeerException("上传失败:" + e.getMessage());
            } finally {
                if (tmp != null) {
                    tmp.delete();
                }
            }
        }
        return ApiResponse.ok(urls);
    }


    @PostMapping(value = "file/upload")
    public ApiResponse<List<FileUploadResult>> fileUpload(@RequestParam("files") MultipartFile[] files) {
        List<FileUploadResult> fileList = new ArrayList<>();
        // 判断文件数组长度
        if(files.length <= 0){
            return ApiResponse.fail("请选择文件");
        }
        for(MultipartFile file : files){
            File tmp = null;
            try {
                tmp = File.createTempFile("upload", "");
                file.transferTo(tmp);
                String key = UUID.randomUUID() + "." + FileUtil.extName(file.getOriginalFilename());
                fileList.add(new FileUploadResult().setFileName(file.getOriginalFilename()).setFilePath(fileService.upload(key, tmp)));
            } catch (Exception e) {
                log.error("upload error,", e);
                throw new PeerException("上传失败:" + e.getMessage());
            } finally {
                if (tmp != null) {
                    tmp.delete();
                }
            }
        }
        return ApiResponse.ok(fileList);
    }

}
