package cn.aibreeze.exam.dto;

import cn.aibreeze.exam.util.PeerException;

import java.io.Serializable;


public class P2PResponse<T extends Serializable> implements Serializable {

    private boolean success;

    private T body;

    private String error;



    /**
     * 成功响应
     * @param body
     * @return
     * @param <T>
     */
    public final static <T extends Serializable> P2PResponse<T>  ok(T body) {
        P2PResponse<T> response = new P2PResponse<>();
        response.setSuccess(true);
        response.setError("");
        response.setBody(body);
        return response;
    }

    /**
     * 失败响应
     * @param error
     * @return
     * @param <T>
     */
    public final static <T extends Serializable> P2PResponse<T> fail(String error) {
        P2PResponse<T> response = new P2PResponse<>();
        response.setSuccess(false);
        response.setError(error);
        response.setBody(null);
        return response;
    }

    /**
     * 获取响应结果并自带检查
     * @return
     * @param <T>
     */
    public final T body() {
        if (success) {
            return getBody();
        } else {
            throw new PeerException(this.error);
        }
    }

    public boolean isSuccess() {
        return success;
    }

    public void setSuccess(boolean success) {
        this.success = success;
    }

    public String getError() {
        return error;
    }

    public void setError(String error) {
        this.error = error;
    }

    public T getBody() {
        return body;
    }

    public void setBody(T body) {
        this.body = body;
    }
}
